class DummyClass:
    def __init__(self, num_1, num_2):
        self.num_1 = num_1
        self.num_2 = num_2

    def add_numbers(self):
        return self.num_1 + self.num_2

# TODO: Create a new-style class (inherit from "object") and write tests for that as well